/******************************************************************************
Copyright (c) 2009, Victor Nakoryakov (aka nail.xx)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice,
  this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice,
  this list of conditions and the following disclaimer in the documentation and/or
  other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

******************************************************************************/

var objWMIService = GetObject("winmgmts:\\\\.\\root\\CIMV2")
var minispace = '<span style="font-size: 50%">&nbsp;</span>'

var updatables = []

function init()
{
    System.Gadget.settingsUI = "settings.html"
    System.Gadget.onSettingsClosed = loadSettings
    loadSettings()
    
    initCpu()
    initRam()
    initBattery()
    update()
}

function update()
{
    for (var i = 0; i < updatables.length; ++i)
        updatables[i]();
    window.setTimeout(update, 1500);
}

function initPercentBar(rootElement, valueFunction)
{
    var b = new PercentBar(rootElement, valueFunction)
    updatables.push(function() { b.updateValue() })
}

function initCpu()
{
    function getLoadFunc(core) {
        return function() { return System.Machine.CPUs.item(core).usagePercentage }
    }
    
    function updateClockSpeed() {
        var queryResult = objWMIService.ExecQuery("SELECT CurrentClockSpeed FROM Win32_Processor")
        var clockSpeed = (new Enumerator(queryResult)).item().CurrentClockSpeed
        cpuSpeed.innerHTML = clockSpeed + minispace + 'MHz'
    }
    
    cpuInfo.innerHTML = System.Machine.CPUs.count + System.Machine.processorArchitecture
    updatables.push(updateClockSpeed)
    initPercentBar(core1Bar, getLoadFunc(0))
    initPercentBar(core2Bar, getLoadFunc(1))
}

function initRam()
{
    function getAvailablePercentage() {
        return 100.0 * System.Machine.availableMemory / System.Machine.totalMemory
    }
    
    function updateAvailabilityText() {
        ramAvailability.innerHTML = System.Machine.availableMemory + minispace + 'MB / ' +
            System.Machine.totalMemory + minispace + 'MB'
    }
    
    updatables.push(updateAvailabilityText)
    initPercentBar(ramBar, getAvailablePercentage)
}

function initBattery()
{
    function getChargePercentage() {
        var queryResult = objWMIService.ExecQuery("SELECT EstimatedChargeRemaining FROM Win32_Battery");
        var chargeRemaining = (new Enumerator(queryResult)).item().EstimatedChargeRemaining
        return Math.min(100, chargeRemaining)
    }
    
    function getTimeLeftText() {
        var queryResult = objWMIService.ExecQuery("SELECT EstimatedRunTime FROM Win32_Battery");
        var runtime = (new Enumerator(queryResult)).item().EstimatedRunTime
        if (runtime == null || runtime > 24 * 60)
            return 'Calculating time remaning...'
        var hours = Math.floor(runtime / 60)
        var minutes = runtime - (Math.floor(runtime / 60) * 60)
        if (minutes < 10)
            minutes = '0' + minutes
        return hours + ':' + minutes + ' left'
    }
    
    function updateText() {
        batteryStat.innerHTML = System.Machine.PowerStatus.isPowerLineConnected ?
            'AC Powered' : getTimeLeftText()
    }
    
    updatables.push(updateText)
    initPercentBar(batteryBar, getChargePercentage)
}

// ============================================================================
function PercentBar(rootElement, valueFunction)
{
    this.valueFunction = valueFunction
    this.innerBar = document.createElement('div')
    this.innerBar.className = 'innerBar'
    this.innerBar.style.width = '50%'
    $('.bar', rootElement).append(this.innerBar)
    this.percentageText = $('.percentage', rootElement)
    
    this.updateValue = function()
    {
        var v = Math.floor(this.valueFunction()).toString() + '%'
        this.innerBar.style.width = v
        this.percentageText.text(v)
    }
}

// ============================================================================
function loadSettings()
{
    theme = System.Gadget.Settings.read('theme')
    if (!theme)
        theme = 'silver'
    setTheme(theme)
}

function setTheme(name)
{
    System.Gadget.background = 'img/bg-' + name + '.png';
    $('body').attr('class', name)
}

